@@ -123,9 +123,6 @@ static const int signals[] = {
 #ifdef SIGFPE
 	SIGFPE,
 #endif
-#ifdef SIGSEGV
-	SIGSEGV,
-#endif
 #ifdef SIGTERM
 	SIGTERM,
 #endif
@@ -265,7 +262,7 @@ static char *get_pid_comm(const pid_t pid)
 		return NULL;
 	}
 	(void)close(fd);
-	buffer[ret-1] = '\0';
+	buffer[ret - 1] = '\0';
 
 	return strdup(buffer);
 }
@@ -369,7 +366,7 @@ static proc_info_t *proc_cache_add_at_hash_index(
 		return NULL;
 	}
 
-	p->pid  = pid;
+	p->pid = pid;
 	p->cmdline = get_pid_cmdline(pid);
 	if (p->cmdline == NULL)
 		p->kernel_thread = true;
@@ -430,62 +427,28 @@ static void proc_cache_cleanup(void)
 }
 
 /*
- *  timeval_double
+ *  timeval_to_double
  *      timeval to a double
  */
-static inline double timeval_double(const struct timeval *tv)
+static inline double timeval_to_double(const struct timeval *tv)
 {
 	return (double)tv->tv_sec + ((double)tv->tv_usec / 1000000.0);
 }
 
 /*
- *  timeval_sub()
- *	timeval a - b
+ *  double_to_timeval
+ *      seconds in double to timeval
  */
-static struct timeval timeval_sub(const struct timeval *a, const struct timeval *b)
+static inline struct timeval double_to_timeval(const double val)
 {
-	struct timeval ret, _b;
+	struct timeval tv;
 
-	_b.tv_sec = b->tv_sec;
-	_b.tv_usec = b->tv_usec;
+	tv.tv_sec = val;
+	tv.tv_usec = (val - (time_t)val) * 1000000.0;
 
-	if (a->tv_usec < _b.tv_usec) {
-		suseconds_t nsec = ((_b.tv_usec - a->tv_usec) / 1000000) + 1;
-		_b.tv_sec += nsec;
-		_b.tv_usec -= (1000000 * nsec);
-	}
-	if (a->tv_usec - _b.tv_usec > 1000000) {
-		suseconds_t nsec = (a->tv_usec - _b.tv_usec) / 1000000;
-		_b.tv_sec -= nsec;
-		_b.tv_usec += (1000000 * nsec);
-	}
-
-	ret.tv_sec = a->tv_sec - _b.tv_sec;
-	ret.tv_usec = a->tv_usec - _b.tv_usec;
-
-	return ret;
+	return tv;
 }
 
-/*
- *  timeval_add()
- *	timeval a + b
- */
-static struct timeval timeval_add(const struct timeval *a, const struct timeval *b)
-{
-	struct timeval ret;
-
-	ret.tv_sec = a->tv_sec + b->tv_sec;
-	ret.tv_usec = a->tv_usec + b->tv_usec;
-	if (ret.tv_usec > 1000000) {
-		int nsec = (ret.tv_usec / 1000000);
-		ret.tv_sec += nsec;
-		ret.tv_usec -= (1000000 * nsec);
-	}
-
-	return ret;
-}
-
-
 static inline unsigned long hash_uid(const uid_t uid)
 {
         unsigned long h = (unsigned long)uid;
@@ -560,19 +523,23 @@ static void uname_cache_cleanup(void)
  *  mem_get_size()
  *	parse proc sizes in K bytes
  */
-static int mem_get_size(FILE *fp, const char *field, uint64_t *size)
+static int mem_get_size(FILE *fp, const char *field, const size_t len, uint64_t *size)
 {
-	char tmp[4096];
+	char buf[4096];
 	uint64_t size_k;
 
 	*size = 0;
 
-	while (!feof(fp)) {
-		if (fscanf(fp, "%4095[^:]: %" SCNi64 "%*[^\n]%*c", tmp, &size_k) == 2) {
-			if (strcmp(tmp, field) == 0) {
-				*size = size_k * 1024;
-				return 0;
-			}
+	/*
+	 *  scanf is expensive, so.. read a line in at a time
+	 *  and if we have a potential match then parse with
+	 *  sscanf
+	 */
+	while (fgets(buf, sizeof(buf) - 1, fp)) {
+		if (!strncmp(buf, field, len) &&
+		    sscanf(buf + len, "%" SCNi64, &size_k) == 1) {
+			*size = size_k * 1024;
+			return 0;
 		}
 	}
 	return -1;
@@ -604,15 +571,15 @@ static int mem_get_entry(FILE *fp, mem_info_t *mem)
 		}
 	}
 
-	if (mem_get_size(fp, "Rss", &rss) < 0)
+	if (mem_get_size(fp, "Rss:", 4, &rss) < 0)
 		return -1;
-	if (mem_get_size(fp, "Pss", &pss) < 0)
+	if (mem_get_size(fp, "Pss:", 4, &pss) < 0)
 		return -1;
-	if (mem_get_size(fp, "Private_Clean", &priv_clean) < 0)
+	if (mem_get_size(fp, "Private_Clean:", 14, &priv_clean) < 0)
 		return -1;
-	if (mem_get_size(fp, "Private_Dirty", &priv_dirty) < 0)
+	if (mem_get_size(fp, "Private_Dirty:", 14, &priv_dirty) < 0)
 		return -1;
-	if (mem_get_size(fp, "Swap", &swap) < 0)
+	if (mem_get_size(fp, "Swap:", 5, &swap) < 0)
 		return -1;
 
 	mem->rss += rss;
@@ -1121,10 +1088,10 @@ static void pid_list_cleanup(void)
  */
 static int parse_pid_list(char *arg)
 {
-	char *str, *token, *saveptr = NULL;
+	char *str, *token;
 	pid_list_t *p;
 
-	for (str = arg; (token = strtok_r(str, ",", &saveptr)) != NULL; str = NULL) {
+	for (str = arg; (token = strtok(str, ",")) != NULL; str = NULL) {
 		if (isdigit(token[0])) {
 			pid_t pid;
 
@@ -1197,8 +1164,8 @@ int main(int argc, char **argv)
 
 	char *json_filename = NULL;
 	FILE *json_file = NULL;
-	double duration_secs = 1.0;
-	struct timeval tv1, tv2, duration, whence;
+	double duration = 1.0, whence = 0.0;
+	struct timeval tv1, tv2;
 	bool forever = true;
 	long int count = 0;
 	size_t npids;
@@ -1259,12 +1226,12 @@ int main(int argc, char **argv)
 
 	if (optind < argc) {
 		errno = 0;
-		duration_secs = strtof(argv[optind++], NULL);
+		duration = strtof(argv[optind++], NULL);
 		if (errno) {
 			fprintf(stderr, "Invalid or out of range value for duration\n");
 			exit(EXIT_FAILURE);
 		}
-		if (duration_secs < 1.0) {
+		if (duration < 1.0) {
 			fprintf(stderr, "Duration must be 1.0 or more seconds.\n");
 			exit(EXIT_FAILURE);
 		}
@@ -1312,10 +1279,6 @@ int main(int argc, char **argv)
 			goto free_cache;
 		mem_cache_prealloc((npids * 5) / 4);
 
-		duration.tv_sec = (time_t)duration_secs;
-		duration.tv_usec = (suseconds_t)(duration_secs * 1000000.0) - (duration.tv_sec * 1000000);
-		whence.tv_sec = 0;
-		whence.tv_usec = 0;
 		if (gettimeofday(&tv1, NULL) < 0) {
 			fprintf(stderr, "gettimeofday failed: errno=%d (%s)\n",
 				errno, strerror(errno));
@@ -1343,6 +1306,7 @@ int main(int argc, char **argv)
 
 		while (!stop_smemstat && (forever || count--)) {
 			struct timeval tv;
+			double t;
 			int ret;
 
 			if (gettimeofday(&tv2, NULL) < 0) {
@@ -1351,21 +1315,17 @@ int main(int argc, char **argv)
 				exit(EXIT_FAILURE);
 			}
 
-			tv = timeval_add(&duration, &whence);
-			tv = timeval_add(&tv, &tv1);
-			tv2 = tv = timeval_sub(&tv, &tv2);
-
-			/* Play catch-up, probably been asleep */
-			if (tv.tv_sec < 0) {
-				tv.tv_sec = 0;
-				tv.tv_usec = 0;
-				tv2 = tv;
+			t = duration + whence + timeval_to_double(&tv1) - timeval_to_double(&tv2);
+			if (t < 0.0) {
+				/* Play catch-up, probably been asleep */
+				t = 0.0;
 			}
+			tv2 = tv = double_to_timeval(t);
 
 			ret = select(0, NULL, NULL, NULL, &tv2);
 			if (ret < 0) {
 				if (errno == EINTR) {
-					duration = timeval_sub(&tv, &tv2);
+					duration = timeval_to_double(&tv) - timeval_to_double(&tv2);
 					stop_smemstat = true;
 				} else {
 					fprintf(stderr, "Select failed: %s\n", strerror(errno));
@@ -1375,13 +1335,13 @@ int main(int argc, char **argv)
 
 			if (mem_get_all_pids(&mem_info_new, &npids) < 0)
 				goto free_cache;
-			mem_dump_diff(json_file, mem_info_old, mem_info_new, timeval_double(&duration));
+			mem_dump_diff(json_file, mem_info_old, mem_info_new, duration);
 
 			mem_cache_free_list(mem_info_old);
 			mem_info_old = mem_info_new;
 			mem_info_new = NULL;
 
-			whence = timeval_add(&duration, &whence);
+			whence += duration;
 		}
 		mem_report_size();
 
